@@ -9,7 +9,7 @@
 #             Warren Moore (zypper package)               <warren@wamonite.com>
 # -----------------------------------------------------------------------------
 # Creation  : 26-Apr-2010
-# Last mod  : 15-Mar-2013
+# Last mod  : 17-Apr-2013
 # -----------------------------------------------------------------------------
 
 """
@@ -40,10 +40,11 @@ See also:
 """
 
 from __future__ import with_statement
-import base64, hashlib, os, re, string, tempfile, subprocess, types, functools, StringIO
+import base64, hashlib, os, re, string, tempfile, subprocess, types
+import tempfile, functools, StringIO
 import fabric, fabric.api, fabric.operations, fabric.context_managers
 
-VERSION               = "0.5.6"
+VERSION               = "0.5.7"
 RE_SPACES             = re.compile("[\s\t]+")
 MAC_EOL               = "\n"
 UNIX_EOL              = "\n"
@@ -55,7 +56,7 @@ OPTION_PACKAGE        = "CUISINE_OPTION_PACKAGE"
 OPTION_PYTHON_PACKAGE = "CUISINE_OPTION_PYTHON_PACKAGE"
 
 AVAILABLE_OPTIONS = dict(
-	package=["apt", "yum", "zypper", "pacman"],
+	package=["apt", "yum", "zypper", "pacman", "emerge"],
 	python_package=["easy_install","pip"]
 )
 
@@ -131,7 +132,7 @@ def is_sudo():   return mode(MODE_SUDO)
 # =============================================================================
 
 def select_package( selection=None ):
-	"""Selects the type of package subsystem to use (ex:apt, yum, zypper, or pacman)."""
+	"""Selects the type of package subsystem to use (ex:apt, yum, zypper, pacman, or emerge)."""
 	supported = AVAILABLE_OPTIONS["package"]
 	if not (selection is None):
 		assert selection in supported, "Option must be one of: %s"  % (supported)
@@ -377,16 +378,10 @@ def file_is_dir(location):
 def file_is_link(location):
 	return run("test -L '%s' && echo OK ; true" % (location)).endswith("OK")
 
-def file_attribs(location, mode=None, owner=None, group=None, recursive=False):
+def file_attribs(location, mode=None, owner=None, group=None):
 	"""Updates the mode/owner/group for the remote file at the given
 	location."""
-	recursive = recursive and "-R " or ""
-	if mode:
-		run('chmod %s %s "%s"' % (recursive, mode,  location))
-	if owner:
-		run('chown %s %s "%s"' % (recursive, owner, location))
-	if group:
-		run('chgrp %s %s "%s"' % (recursive, group, location))
+	return dir_attribs(location, mode, owner, group, False)
 
 def file_attribs_get(location):
 	"""Return mode, owner, and group for remote path.
@@ -466,7 +461,7 @@ def file_upload(remote, local, sudo=None, scp=False):
 	if not file_exists(remote) or sig != file_md5(remote):
 		if is_local():
 			if use_sudo:
-				sudo('cp "%s" "%s"'%(local,remote))
+				globals()['sudo']('cp "%s" "%s"'%(local,remote))
 			else:
 				run('cp "%s" "%s"'%(local,remote))
 		else:
@@ -581,7 +576,12 @@ def process_kill(name, signal=9, exact=False):
 
 def dir_attribs(location, mode=None, owner=None, group=None, recursive=False):
 	"""Updates the mode/owner/group for the given remote directory."""
-	file_attribs(location, mode, owner, group, recursive)
+	if mode:
+		run('chmod %s "%s"' % (recursive, mode,  location))
+	if owner:
+		run('chown %s "%s"' % (recursive, owner, location))
+	if group:
+		run('chgrp %s "%s"' % (recursive, group, location))
 
 def dir_exists(location):
 	"""Tells if there is a remote directory at the given location."""
@@ -650,24 +650,24 @@ def repository_ensure_apt(repository):
 
 def package_update_apt(package=None):
 	if package == None:
-		sudo("apt-get --yes update")
+		sudo("apt-get -q --yes update")
 	else:
 		if type(package) in (list, tuple):
 			package = " ".join(package)
-		sudo('DEBIAN_FRONTEND=noninteractive apt-get --yes -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold" upgrade ' + package)
+		sudo('DEBIAN_FRONTEND=noninteractive apt-get -q --yes -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold" upgrade ' + package)
 
 def package_upgrade_apt(distupgrade=False):
 	if distupgrade:
-		sudo('DEBIAN_FRONTEND=noninteractive apt-get --yes -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold" dist-upgrade')
+		sudo('DEBIAN_FRONTEND=noninteractive apt-get -q --yes -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold" dist-upgrade')
 	else:
-		sudo('DEBIAN_FRONTEND=noninteractive apt-get --yes -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold" upgrade')
+		sudo('DEBIAN_FRONTEND=noninteractive apt-get -q --yes -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold" upgrade')
 
 def package_install_apt(package, update=False):
 	if update:
-		sudo('DEBIAN_FRONTEND=noninteractive apt-get --yes -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold" update')
+		sudo('DEBIAN_FRONTEND=noninteractive apt-get -q --yes -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold" update')
 	if type(package) in (list, tuple):
 		package = " ".join(package)
-	sudo("DEBIAN_FRONTEND=noninteractive apt-get --yes install %s" % (package))
+	sudo("DEBIAN_FRONTEND=noninteractive apt-get -q --yes install %s" % (package))
 
 def package_ensure_apt(package, update=False):
 	"""Ensure apt packages are installed"""
@@ -685,12 +685,12 @@ def package_ensure_apt(package, update=False):
 def package_clean_apt(package=None):
 	if type(package) in (list, tuple):
 		package = " ".join(package)
-	sudo("DEBIAN_FRONTEND=noninteractive apt-get -y --purge remove %s" % package)
+	sudo("DEBIAN_FRONTEND=noninteractive apt-get -q -y --purge remove %s" % package)
 
 def package_remove_apt(package, autoclean=False):
-	sudo('DEBIAN_FRONTEND=noninteractive apt-get --yes -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold" remove ' + package)
+	sudo('DEBIAN_FRONTEND=noninteractive apt-get -q --yes -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold" remove ' + package)
 	if autoclean:
-		sudo('apt-get --yes autoclean')
+		sudo('apt-get -q --yes autoclean')
 
 # -----------------------------------------------------------------------------
 # YUM PACKAGE (RedHat, CentOS)
@@ -822,6 +822,59 @@ def package_remove_pacman(package, autoclean=False):
 	else:
 		sudo('pacman --noconfirm -R ' + package)
 
+
+# -----------------------------------------------------------------------------
+# EMERGE PACKAGE (Gentoo Portage)
+# added by davidmmiller - 20130417 - v0.1 (status - works for me...)
+# -----------------------------------------------------------------------------
+
+def repository_ensure_emerge(repository):
+	raise Exception("Not implemented for emerge")
+	"""This will be used to add Portage overlays in a future update."""
+
+def package_upgrade_emerge(distupgrade=False):
+		sudo("emerge -q --update --deep --newuse --with-bdeps=y world")
+
+def package_update_emerge(package=None):
+	if package == None:
+		sudo("emerge -q --sync")
+	else:
+		if type(package) in (list, tuple):
+			package = " ".join(package)
+		sudo("emerge -q --update --newuse %s" % package)
+
+def package_install_emerge(package, update=False):
+	if update:
+		sudo("emerge -q --sync")
+	if type(package) in (list, tuple):
+		package = " ".join(package)
+	sudo("emerge -q %s" % (package))
+
+def package_ensure_emerge(package, update=False):
+	if not isinstance(package, basestring):
+		package = " ".join(package)
+	if update:
+		sudo("emerge -q --update --newuse %s" % package)
+	else:
+		sudo("emerge -q --noreplace %s" % package)
+
+def package_clean_emerge(package=None):
+	if type(package) in (list, tuple):
+		package = " ".join(package)
+	if package:
+		sudo("CONFIG_PROTECT='-*' emerge --quiet-unmerge-warn --unmerge %s" % package)
+	else:
+		sudo('emerge -q --depclean')
+		sudo('revdep-rebuild -q')
+
+def package_remove_emerge(package, autoclean=False):
+	if autoclean:
+		sudo('emerge --quiet-unmerge-warn --unmerge ' + package)
+		sudo('emerge -q --depclean')
+		sudo('revdep-rebuild -q')
+	else:
+		sudo('emerge --quiet-unmerge-warn --unmerge ' + package)
+
 # =============================================================================
 #
 # PYTHON PACKAGE OPERATIONS
@@ -867,7 +920,7 @@ def python_package_upgrade_pip(package,E=None):
 		E='-E %s' %E
 	else:
 		E=''
-	run('pip upgrade %s %s' %(E,package))
+	run('pip install --upgrade %s %s' %(E,package))
 
 def python_package_install_pip(package=None,r=None,pip=None):
 	'''
@@ -896,7 +949,7 @@ def python_package_ensure_pip(package=None, r=None, pip=None):
 	path to a virtualenv. If provided, it will be added to the pip call.
 	'''
 	#FIXME: At the moment, I do not know how to check for the existence of a pip package and
-	# I am not sure if this really makes sense, based on the pip built in functionality. 
+	# I am not sure if this really makes sense, based on the pip built in functionality.
 	# So I just call the install functions
 	pip=pip or fabric.api.env.get('pip','pip')
 	python_package_install_pip(package,r,pip)
@@ -908,7 +961,7 @@ def python_package_remove_pip(package, E=None, pip=None):
 	is equivalent to the "-r" parameter of pip.
 	Either "package" or "r" needs to be provided
 	The optional argument "E" is equivalent to the "-E" parameter of pip. E is the
-	path to a virtualenv. If provided, it will be added to the pip call. 
+	path to a virtualenv. If provided, it will be added to the pip call.
 	'''
 	pip=pip or fabric.api.env.get('pip','pip')
 	return run('%s uninstall %s' %(pip,package))
@@ -934,7 +987,7 @@ def python_package_ensure_easy_install(package):
 	The "package" argument, defines the name of the package that will be ensured.
 	'''
 	#FIXME: At the moment, I do not know how to check for the existence of a py package and
-	# I am not sure if this really makes sense, based on the easy_install built in functionality. 
+	# I am not sure if this really makes sense, based on the easy_install built in functionality.
 	# So I just call the install functions
 	python_package_install_easy_install(package)
 
@@ -943,7 +996,7 @@ def python_package_remove_easy_install(package):
 	The "package" argument, defines the name of the package that will be removed.
 	'''
 	#FIXME: this will not remove egg file etc.
-	run('easy_install -m %s' %package) 
+	run('easy_install -m %s' %package)
 
 # =============================================================================
 #
@@ -982,10 +1035,11 @@ def user_passwd(name, passwd, encrypted_passwd=True):
 		sudo("echo %s | openssl base64 -A -d | chpasswd" % (encoded_password))
 
 def user_create(name, passwd=None, home=None, uid=None, gid=None, shell=None,
-	uid_min=None, uid_max=None, encrypted_passwd=True, fullname=None):
+	uid_min=None, uid_max=None, encrypted_passwd=True, fullname=None, createhome=True):
 	"""Creates the user with the given name, optionally giving a
 	specific password/home/uid/gid/shell."""
-	options = ["-m"]
+	options = []
+
 	if home:
 		options.append("-d '%s'" % (home))
 	if uid:
@@ -1002,30 +1056,36 @@ def user_create(name, passwd=None, home=None, uid=None, gid=None, shell=None,
 	if uid_max:
 		options.append("-K UID_MAX='%s'" % (uid_max))
 	if fullname:
-		options.append("--gecos='%s'" % (fullname))
+		options.append("-c '%s'" % (fullname))
+	if createhome:
+		options.append("-m")
 	sudo("useradd %s '%s'" % (" ".join(options), name))
 	if passwd:
 		user_passwd(name=name,passwd=passwd,encrypted_passwd=encrypted_passwd)
 
-def user_check(name=None, uid=None):
+def user_check(name=None, uid=None, need_passwd=True):
 	"""Checks if there is a user defined with the given name,
 	returning its information as a
 	'{"name":<str>,"uid":<str>,"gid":<str>,"home":<str>,"shell":<str>}'
-	or 'None' if the user does not exists."""
+	or 'None' if the user does not exists.
+	need_passwd (Boolean) indicates if password to be included in result or not.
+		If set to True it parses 'getent shadow' and needs sudo access
+	"""
 	assert name!=None or uid!=None,     "user_check: either `uid` or `name` should be given"
 	assert name is None or uid is None,"user_check: `uid` and `name` both given, only one should be provided"
 	if   name != None:
-		d = sudo("cat /etc/passwd | egrep '^%s:' ; true" % (name))
+		d = run("getent passwd | egrep '^%s:' ; true" % (name))
 	elif uid != None:
-		d = sudo("cat /etc/passwd | egrep '^.*:.*:%s:' ; true" % (uid))
+		d = run("getent passwd | egrep '^.*:.*:%s:' ; true" % (uid))
 	results = {}
 	s = None
 	if d:
 		d = d.split(":")
-		assert len(d) >= 7, "/etc/passwd entry is expected to have at least 7 fields, got %s in: %s" % (len(d), ":".join(d))
+		assert len(d) >= 7, "passwd entry returned by getent is expected to have at least 7 fields, got %s in: %s" % (len(d), ":".join(d))
 		results = dict(name=d[0], uid=d[2], gid=d[3], fullname=d[4], home=d[5], shell=d[6])
-		s = sudo("cat /etc/shadow | egrep '^%s:' | awk -F':' '{print $2}'" % (results['name']))
-		if s: results['passwd'] = s
+		if need_passwd:
+			s = sudo("getent shadow | egrep '^%s:' | awk -F':' '{print $2}'" % (results['name']))
+			if s: results['passwd'] = s
 	if results:
 		return results
 	else:
@@ -1080,7 +1140,7 @@ def group_check(name):
 	returning its information as a
 	'{"name":<str>,"gid":<str>,"members":<list[str]>}' or 'None' if
 	the group does not exists."""
-	group_data = run("cat /etc/group | egrep '^%s:' ; true" % (name))
+	group_data = run("getent group | egrep '^%s:' ; true" % (name))
 	if group_data:
 		name, _, gid, members = group_data.split(":", 4)
 		return dict(name=name, gid=gid,
@@ -1123,9 +1183,11 @@ def group_user_del(group, user):
 		"""remove the given user from the given group."""
 		assert group_check(group), "Group does not exist: %s" % (group)
 		if group_user_check(group, user):
-				group_for_user = run("cat /etc/group | egrep -v '^%s:' | grep '%s' | awk -F':' '{print $1}' | grep -v %s; true" % (group, user, user)).splitlines()
+				group_for_user = run("getent group | egrep -v '^%s:' | grep '%s' | awk -F':' '{print $1}' | grep -v %s; true" % (group, user, user)).splitlines()
 				if group_for_user:
 						sudo("usermod -G '%s' '%s'" % (",".join(group_for_user), user))
+				else:
+						sudo("usermod -G '' '%s'" % (user))
 
 # =============================================================================
 #
@@ -1152,7 +1214,7 @@ def ssh_keygen(user, keytype="dsa"):
 def ssh_authorize(user, key):
 	"""Adds the given key to the '.ssh/authorized_keys' for the given
 	user."""
-	d = user_check(user)
+	d = user_check(user, need_passwd=False)
 	group = d["gid"]
 	keyf = d["home"] + "/.ssh/authorized_keys"
 	if key[-1] != "\n":
@@ -1170,6 +1232,27 @@ def ssh_authorize(user, key):
 		file_write(keyf, key,             owner=user, group=group, mode="600")
 		return False
 
+def ssh_unauthorize(user, key):
+	"""Removes the given key to the '.ssh/authorized_keys' for the given
+	user."""
+	d = user_check(user, need_passwd=False)
+	group = d["gid"]
+	keyf = d["home"] + "/.ssh/authorized_keys"
+	if file_exists(keyf):
+		tmpfile = tempfile.NamedTemporaryFile()
+		fabric.operations.get(keyf, tmpfile.name)
+		keys = [line.strip() for line in tmpfile]
+		tmpfile.close()
+		if key in keys:
+			tmpfile = tempfile.NamedTemporaryFile()
+			keys.remove(key)
+			content = '\n'.join(keys) + '\n'
+			tmpfile.write(content)
+			tmpfile.flush()
+			fabric.operations.put(tmpfile.name, keyf, mode=0600)
+			tmpfile.close()
+	return True
+
 # =============================================================================
 #
 # UPSTART
